import { BehaviorSubject, Observable } from 'rxjs';
import type { Metadata, SchemaResponse } from '../metadata';
import { MetaCache } from '../metadata';
import { ConnectEventsEmitter } from './connect.events';
import { ConnectConfig } from './connect.types';
import { CommitParams, Message, MessageBuilder, RequestParams } from './message';
import type { SocketObservable } from './socket';
import { Socket } from './socket';
/**
 * The default configuration for connecting to a WS service.
 * @public
 */
export declare const defaultConnectConfig: ConnectConfig;
/**
 * Represents a connection and its capabilities on a remote server
 * Provides methods to send and receive messages, as well as metadata and schema information.
 * @public
 */
export interface Connect {
    /**
     * Indicates whether Connect is working or is idle.
     * @public
     */
    isWorking: boolean;
    /**
     * Indicates whether the connection is currently established.
     * @public
     */
    readonly isConnected: boolean;
    /**
     * An Observable that emits a boolean value indicating whether the connection is currently established.
     * @public
     */
    readonly isConnected$?: Observable<boolean>;
    /**
     * A behavior subject holding the current connection status.
     * @deprecated - Use `isConnected$` instead.
     * @public
     */
    readonly isConnectedSubject?: BehaviorSubject<boolean>;
    /**
     * Sends a message to the server and returns a promise that resolves with the server's response.
     * @internal
     * @param message - The message to send.
     * @typeParam T - The type of the Message's DETAILS payload.
     * @param needsHandling - Whether the response needs to be handled manually. Default: false.
     * @returns A promise that resolves with the server's response.
     */
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    /**
     * Returns whether the connection is in HTTP mode.
     * @internal
     * @returns True if the connection is in HTTP mode; false otherwise.
     */
    httpMode(): boolean;
    /**
     * Connects to a server at the specified host.
     * @public
     * @param host - The host to connect to.
     * @returns A promise that resolves with a boolean indicating whether the connection was successful.
     */
    connect(host: string): Promise<boolean>;
    /**
     * Disconnects from the server.
     * @public
     */
    disconnect(): void;
    /**
     * Sets the validity of the current session.
     * Use in Auth scenarios where session expires or renews, but the connection is still active.
     * @internal
     * @param valid - Whether the session is valid.
     */
    setValidSession(valid: boolean): void;
    /**
     * Retrieves a snapshot of data for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the snapshot for.
     * @param params - Additional parameters to pass to the server.
     * @returns A promise that resolves with the snapshot data.
     */
    snapshot(resourceName: string, params?: any): Promise<Message>;
    /**
     * Starts listening for updates on the specified resource.
     * @public
     * @param resourceName - The name of the resource to listen for updates on.
     * @param onMessage - A callback function to handle received messages.
     * @param onError - A callback function to handle errors.
     * @param params - Additional parameters to pass to the server.
     * @returns An observable that emits messages received from the server.
     */
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    /**
     * Requests additional rows of data for the specified source reference.
     * @public
     * @param sourceRef - The source reference to request more rows for.
     * @returns A promise that resolves with the additional rows of data.
     */
    getMoreRows(sourceRef: string): Promise<Message>;
    /**
     * Requests additional columns for the specified source reference.
     * @public
     * @param sourceRef - The source reference to request more columns for.
     * @returns A promise that resolves with the additional columns.
     */
    getMoreColumns(sourceRef: string): Promise<Message>;
    /**
     * Logs off from the specified data source.
     * @public
     * @param sourceRef - The reference of the data source to log off from.
     * @returns A promise that resolves with a message indicating success or failure.
     */
    dataLogoff(sourceRef: string): Promise<Message>;
    /**
     * Retrieves the metadata for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the metadata for.
     * @param useCache - Whether to use the cached metadata if available. Default: true.
     * @returns A promise that resolves with the metadata.
     */
    getMetadata(resourceName: string, useCache?: boolean): Promise<Metadata>;
    /**
     * Retrieves the JSON schema for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the schema for.
     */
    getJSONSchema(resourceName: string): Promise<SchemaResponse>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    /**
     * Sends a request to the server to get available resources.
     * @public
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response with available resources.
     */
    getAvailableResources(params?: RequestParams): Promise<Message>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
}
/**
 * The default implementation for the Connect interface (WS-only).
 * @public
 */
export declare class DefaultConnect implements Connect {
    socket: Socket;
    private messageBuilder;
    private metaCache;
    private events;
    private config;
    isWorking: boolean;
    get isConnected(): boolean;
    get isConnectedSubject(): BehaviorSubject<boolean>;
    get isConnected$(): Observable<boolean>;
    constructor(socket: Socket, messageBuilder: MessageBuilder, metaCache: MetaCache, events: ConnectEventsEmitter, config?: ConnectConfig);
    httpMode(): boolean;
    setValidSession(valid: boolean): void;
    send<T>(message: Message<any>, needsHandling?: boolean): Promise<any>;
    connect(host: string): Promise<boolean>;
    disconnect(): void;
    snapshot(resourceName: string, params?: any): Promise<Message>;
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    getMoreRows(sourceRef: string): Promise<Message>;
    getMoreColumns(sourceRef: string): Promise<Message>;
    dataLogoff(streamSourceRef: string): Promise<Message>;
    getMetadata(resourceName: string, useCache?: boolean): Promise<Metadata>;
    getJSONSchema(resourceName: string, useCache?: boolean): Promise<SchemaResponse>;
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    getAvailableResources(params?: any): Promise<Message>;
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
}
/**
 * The DI token for the Connect interface (WS-only).
 * @internal
 */
export declare const WSConnect: import("@microsoft/fast-foundation").InterfaceSymbol<Connect>;
/**
 * The DI token for the Connect interface (HTTP | WS).
 * @internal
 */
export declare const Connect: import("@microsoft/fast-foundation").InterfaceSymbol<Connect>;
/**
 * Get Connect from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getConnect } from '@genesislcap/foundation-comms';
 * import { API_HOST } from '@genesislcap/foundation-utils';
 * ...
 * private connect = getConnect();
 * ...
 * const connected = await this.connect.connect(API_HOST); // < WebSocket url
 * ```
 *
 * @public
 */
export declare function getConnect(): Connect;
//# sourceMappingURL=connect.d.ts.map