import { ConnectEvents } from '../connect/connect.events';
import { SerializedSocketStatus, SocketStatus } from '../connect/socket.status';
import { User } from '../user/user';
import { NetworkMonitorConfig } from './config';
import { MonitoredResource, MonitoredResourceMetadata, ResourceHealth, SystemHealthStatus } from './types';
/**
 * @public
 */
export declare const networkLogger: import("@genesislcap/foundation-logger").Logger;
/**
 * `NetworkMonitor` monitors network and resource availability and health.
 *
 * @remarks
 * UIs can listen for 'system-health-changed' events and display visual overlays to communicate status to the user.
 *
 * @public
 */
export interface NetworkMonitor {
    status: SystemHealthStatus;
    create(): void;
    destroy(): void;
    /**
     * @param source - The source of the call, which may have been an event or scheduled interval.
     * @param force - A flag to bypass abort if already in progress logic.
     */
    runAllChecks(source?: unknown, force?: boolean): Promise<void>;
    /**
     * @param resource - The resource reference.
     * @param metadata - The resource metadata.
     */
    addResource(resource: MonitoredResource, metadata: MonitoredResourceMetadata): void;
    /**
     * @param resource - The resource reference.
     */
    removeResource(resource: MonitoredResource): void;
}
/**
 * Default NetworkMonitor implementation.
 * @public
 */
export declare class DefaultNetworkMonitor implements NetworkMonitor {
    /**
     * High-level configuration.
     * @internal
     */
    protected config: NetworkMonitorConfig;
    /**
     * @internal
     */
    protected socketStatus: SocketStatus;
    /**
     * @internal
     */
    protected connectEvents: ConnectEvents;
    /**
     * @internal
     */
    protected user: User;
    /**
     * @internal
     */
    protected unsubscribableObservers: any[];
    /**
     * @privateRemarks
     * Originally aimed to use WeakMap for resources, but need to iterate and process these. I also had a
     * `this.registry = new FinalizationRegistry<any>(this.registryCleanup);` in the mix to handle clean up actions.
     *
     * @internal
     */
    protected resources: Map<MonitoredResource, MonitoredResourceMetadata>;
    /**
     * @internal
     */
    protected internalId: ReturnType<typeof setInterval>;
    /**
     * @internal
     */
    protected isRunningChecks: boolean;
    /**
     * {@inheritDoc NetworkMonitor.status}
     */
    status: SystemHealthStatus;
    /**
     * {@inheritDoc NetworkMonitor.create}
     */
    create(): void;
    /**
     * {@inheritDoc NetworkMonitor.destroy}
     */
    destroy(): void;
    /**
     * @internal
     */
    protected isOnlineCheck(): Promise<boolean>;
    /**
     * {@inheritDoc NetworkMonitor.runAllChecks}
     */
    runAllChecks: (source?: unknown, force?: boolean) => Promise<void>;
    /**
     * {@inheritDoc NetworkMonitor.addResource}
     */
    addResource(resource: MonitoredResource, metadata: MonitoredResourceMetadata): void;
    /**
     * {@inheritDoc NetworkMonitor.removeResource}
     */
    removeResource(resource: MonitoredResource): void;
    /**
     * @internal
     */
    protected addListeners(): void;
    /**
     * @internal
     */
    protected removeListeners(): void;
    /**
     * @internal
     */
    protected reportSystemHealth(detail: Partial<SystemHealthStatus>): void;
    /**
     * @internal
     */
    protected reportStabilityToGenesis(detail: Partial<SystemHealthStatus>): Promise<boolean>;
    /**
     * @remarks
     * Deal with false positives.
     *
     * @privateRemarks
     * There's an argument here for and against normalising ResourceHealth. I think it's more useful to consumers like
     * visual overlays if there is no conflicting information. If we're offline and using the cache, then that should be
     * reflected here.
     *
     * @internal
     */
    protected normaliseResourceHealth(health: ResourceHealth, isOnline?: boolean): ResourceHealth;
    /**
     * @remarks
     * Deal with false positives.
     *
     * @privateRemarks
     * There's an argument here for and against a normalised SerializedSocketStatus. I think it's more useful to consumers
     * like visual overlays if there is no conflicting information. If we're offline and using the cache, then that
     * should be reflected here.
     *
     * @internal
     */
    protected normaliseSerializedSocketStatus(socketStatus: SerializedSocketStatus, isOnline?: boolean): SerializedSocketStatus;
    /**
     * @internal
     */
    protected normaliseEventDetail(detail: Partial<SystemHealthStatus>): SystemHealthStatus;
    /**
     * @internal
     */
    protected getFormattedTimestamp(milliseconds: number): string;
    /**
     * @internal
     */
    protected getNowTimestamp(): number;
    /**
     * @internal
     */
    protected getNextTimestamp(): number;
    /**
     * @internal
     */
    protected synchroniseCheckInterval(): void;
}
/**
 * NetworkMonitor DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const NetworkMonitor: import("@microsoft/fast-foundation").InterfaceSymbol<NetworkMonitor>;
/**
 * Gets NetworkMonitor from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getNetworkMonitor } from '@genesislcap/foundation-comms';
 * ...
 * private monitor = getNetworkMonitor();
 * ```
 *
 * @public
 */
export declare function getNetworkMonitor(): NetworkMonitor;
//# sourceMappingURL=service.d.ts.map