import { createObserver } from '@genesislcap/foundation-utils';
import { DI } from '@microsoft/fast-foundation';
/**
 * @public
 */
export const CommitEventType = 'connect-commit-event';
/**
 * @public
 */
export const StreamEventType = 'connect-stream-event';
/**
 * @public
 */
export const StreamCompleteEventType = 'connect-stream-complete-event';
/**
 * @public
 */
export const MetadataEventType = 'connect-metadata-event';
/**
 * Default ConnectEvents implementation.
 * @public
 */
export class DefaultConnectEvents {
    constructor() {
        /** @internal */
        this.streamListeners = createObserver();
        /** @internal */
        this.streamCompleteListeners = createObserver();
        /** @internal */
        this.commitListeners = createObserver();
        /** @internal */
        this.metadataListeners = createObserver();
    }
    /** {@inheritDoc ConnectEvents.addStreamListener} */
    addStreamListener(listener, complete) {
        var _a, _b;
        const unsubscribe = (_a = this.streamListeners) === null || _a === void 0 ? void 0 : _a.subscribe(listener);
        const completeUnsubscribe = (_b = this.streamCompleteListeners) === null || _b === void 0 ? void 0 : _b.subscribe(complete);
        return () => {
            unsubscribe();
            completeUnsubscribe();
        };
    }
    /** {@inheritDoc ConnectEventsEmitter.onStreamEvent} */
    onStreamEvent(resourceName, message, stream, functions) {
        var _a;
        (_a = this.streamListeners) === null || _a === void 0 ? void 0 : _a.publish(new CustomEvent(StreamEventType, {
            detail: {
                functions,
                message,
                resourceName,
                stream,
            },
        }));
    }
    /** {@inheritDoc ConnectEventsEmitter.onStreamCompleteEvent} */
    onStreamCompleteEvent(stream) {
        var _a;
        (_a = this.streamCompleteListeners) === null || _a === void 0 ? void 0 : _a.publish(new CustomEvent(StreamCompleteEventType, {
            detail: stream,
        }));
    }
    /** {@inheritDoc ConnectEvents.addCommitListener} */
    addCommitListener(listener) {
        var _a;
        return (_a = this.commitListeners) === null || _a === void 0 ? void 0 : _a.subscribe(listener);
    }
    /** {@inheritDoc ConnectEventsEmitter.onStreamEvent} */
    onCommitEvent(eventName, message) {
        var _a;
        (_a = this.commitListeners) === null || _a === void 0 ? void 0 : _a.publish(new CustomEvent(CommitEventType, {
            detail: {
                eventName,
                message,
            },
        }));
    }
    /** {@inheritDoc ConnectEvents.addMetadataListener} */
    addMetadataListener(listener) {
        var _a;
        return (_a = this.metadataListeners) === null || _a === void 0 ? void 0 : _a.subscribe(listener);
    }
    /** {@inheritDoc ConnectEventsEmitter.onMetadataEvent} */
    onMetadataEvent(resourceName, message, error) {
        var _a;
        (_a = this.metadataListeners) === null || _a === void 0 ? void 0 : _a.publish(new CustomEvent(MetadataEventType, {
            detail: {
                resourceName,
                message,
                error,
            },
        }));
    }
}
/**
 * ConnectEvents DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const ConnectEvents = DI.createInterface((x) => x.singleton(DefaultConnectEvents));
/**
 * ConnectEventsEmitter DI key.
 *
 * @privateRemarks
 * ConnectEventsEmitter is used by Connect.
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const ConnectEventsEmitter = DI.createInterface((x) => x.aliasTo(ConnectEvents));
