import { __awaiter, __decorate, __param } from "tslib";
import { DI, optional } from '@microsoft/fast-foundation';
import { Auth } from '../../auth';
import { Connect } from '../../connect';
import { logger } from '../../utils';
import { defaultKVStorageConfig, KVStorageConfig } from './kv-config.service';
import { KVEventMessageType, KVMessageType, } from './kv.types';
/**
 * The default implementation for the KVStorage interface.
 * @alpha
 */
let DefaultKVStorage = class DefaultKVStorage {
    constructor(connect, auth, kvStorageConfig = defaultKVStorageConfig) {
        this.connect = connect;
        this.auth = auth;
        this.kvStorageConfig = kvStorageConfig;
        this.checkAvailability();
    }
    /**
     * Determine whether KV Storage is available for use.
     * @internal
     */
    checkAvailability() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const allSystemTables = yield this.connect.request('SYSTEM_TABLES');
            const isKVStoreAvailable = !!((_a = allSystemTables === null || allSystemTables === void 0 ? void 0 : allSystemTables.REPLY) === null || _a === void 0 ? void 0 : _a.find((table) => table.TABLE_NAME === 'KEY_VALUE'));
            if (!isKVStoreAvailable) {
                logger.error('KV Storage is not available on the server resources. Update your server to use it.');
            }
        });
    }
    /**
     * Deletes the specified key-value pairs from the KV storage.
     * @alpha
     * @param kvs - The key names to delete.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.delete(['myKey']);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    delete(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                DETAILS: {
                    KEYS: keys,
                },
            };
            const result = yield this.connect.commitEvent(KVEventMessageType.EVENT_KEY_VALUE_DELETE, params);
            if (result && result.MESSAGE_TYPE === 'EVENT_ACK') {
                return true;
            }
            else {
                return { ERROR: result.ERROR, WARNING: result.WARNING };
            }
        });
    }
    /**
     * Gets the specified key-value pairs from the KV storage.
     * @remarks If no key is specified, all key-value pairs for the current user are returned.
     * @alpha
     * @param kv - The single key-value pair to get.
     * @returns A promise that resolves to an array of key-value pairs.
     *
     * @example
     * ```ts
     * ...
     *  const result = await this.kvStorage.get({ key: 'myKey' });
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }]
     *
     *  const result2 = await this.kvStorage.get();
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }, { key: 'myOtherKey', value: 'myOtherValue' }]
     *  ...
     * ```
     */
    get(key) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                REQUEST: {
                    USER_NAME: (_a = this.kvStorageConfig.authEmail) !== null && _a !== void 0 ? _a : (_c = (_b = this.auth) === null || _b === void 0 ? void 0 : _b.currentUser) === null || _c === void 0 ? void 0 : _c.username,
                    KEY: key,
                },
            };
            const result = yield this.connect.request(KVMessageType.REQ_KEY_VALUE_READ, params);
            if (result && result.REPLY && result.REPLY.length > 0) {
                const kvs = result.REPLY.map((kv) => ({ kv: { key: kv.KEY, value: kv.VALUE } }));
                return kvs;
            }
            else {
                return [];
            }
        });
    }
    /**
     * Lists all keys for the user.
     * @alpha
     * @returns A promise that resolves to all keys for the user.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.list();
     *  logger.debug(result); // ['myKey', 'myOtherKey']
     *  ...
     * ```
     */
    list() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                REQUEST: {
                    USER_NAME: (_a = this.kvStorageConfig.authEmail) !== null && _a !== void 0 ? _a : (_c = (_b = this.auth) === null || _b === void 0 ? void 0 : _b.currentUser) === null || _c === void 0 ? void 0 : _c.username,
                },
            };
            const result = yield this.connect.request(KVMessageType.REQ_KEY_VALUE_LIST, params);
            if (result && result.REPLY && result.REPLY.length > 0) {
                const keys = result.REPLY.map((kv) => kv.KEY);
                return keys;
            }
            else {
                return [];
            }
        });
    }
    /**
     * Inserts or updates the specified key-value pairs in the KV storage.
     * @alpha
     * @param kvs - The key-value pairs to insert or update.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.put([{ key: 'myKey', value: 'myValue' }]);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    put(kvs) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                DETAILS: {
                    RECORDS: kvs.map((kv) => ({
                        KEY: kv.key,
                        VALUE: kv.value,
                    })),
                },
            };
            const result = yield this.connect.commitEvent(KVEventMessageType.EVENT_KEY_VALUE_PUT, params);
            if (result && result.MESSAGE_TYPE === 'EVENT_ACK') {
                return true;
            }
            else {
                return { ERROR: result.ERROR, WARNING: result.WARNING };
            }
        });
    }
};
DefaultKVStorage = __decorate([
    __param(0, Connect),
    __param(1, Auth),
    __param(2, optional(KVStorageConfig))
], DefaultKVStorage);
export { DefaultKVStorage };
/**
 * The DI token for the KVStorage interface.
 * @internal
 */
export const KVStorage = DI.createInterface((x) => x.singleton(DefaultKVStorage));
